@@ -1,12 +1,12 @@
 # coding=utf-8
 import copy
 import warnings
-from collections import namedtuple
+from collections import namedtuple, deque
+
+from functools import wraps
 
 from ClickSQL.clickhouse.ClickHouseExt import ClickHouseTableNodeExt
 from ClickSQL.errors import ClickHouseTableNotExistsError
-from functools import wraps
-
 from ClickSQL.nodes.groupby import GroupSQLUtils
 from ClickSQL.nodes.merge import MergeSQLUtils
 
@@ -33,7 +33,22 @@ ft_node = namedtuple('factortable', factor_parameters)
 "add auto-increment col by materialized bitOr(bitShiftLeft(toUInt64(now64()),24), rowNumberInAllBlocks()) "
 
 
-class BaseSingleFactorBaseNode(object):
+class DelayTasks(deque):
+    def __init__(self,_query, *args, **kwargs):
+        super(DelayTasks, self).__init__(*args, **kwargs)
+
+        self._query = _query
+
+    def run(self, no_yield=False):
+        result = (self._query(sql) for sql in self)
+        if no_yield:
+            return list(result)
+
+        else:
+            return result
+
+
+class BaseSingleQueryBaseNode(object):
     """
     init operator
     and set fid_ck,dt_max_1st,execute,no_self_update for update functions
@@ -42,7 +57,7 @@ class BaseSingleFactorBaseNode(object):
     __Name__ = "基础因子库单因子基类"
     __slots__ = (
         'operator', 'db', 'table', 'db_table', '_kwargs', '_raw_kwargs', 'status', '_INFO', 'depend_tables',
-        '_fid_ck', '_dt_max_1st', '_execute', '_no_self_update'
+        '_fid_ck', '_dt_max_1st', '_execute', '_no_self_update', 'delay_tasks'
     )
 
     def __init__(self, src: str, db_table: (None, str) = None, info=None, **kwargs):
@@ -69,13 +84,13 @@ class BaseSingleFactorBaseNode(object):
 
         if db_table is None:
             src_db_table = self.operator.db_table
-            db_table_all, db, table = self.db_split(src_db_table)
+            db_table_all, db, table = self._db_split(src_db_table)
             # if '.' in src_db_table:
             #     self.db_table = src_db_table
             # else:
             #     raise ValueError('db_table parameter get wrong type!')
         elif isinstance(db_table, str):
-            db_table_all, db, table = self.db_split(db_table)
+            db_table_all, db, table = self._db_split(db_table)
         else:
             raise ValueError('db_table only accept str!')
         self.db_table = db_table_all
@@ -88,8 +103,11 @@ class BaseSingleFactorBaseNode(object):
         self.status = 'SQL'
         self._INFO = info
 
+        self.delay_tasks = DelayTasks(self.operator)
+        # self.delay_tasks._query = self.operator
+
     @staticmethod
-    def db_split(src_db_table):
+    def _db_split(src_db_table):
         if isinstance(src_db_table, str):
             if '.' in src_db_table:
                 db, table = src_db_table.split('.')
@@ -134,7 +152,6 @@ class BaseSingleFactorBaseNode(object):
 
     def decorate(self, target_conn_args='conn'):
         def afunc(func):
-
             def _afunc(*args, **kwargs):
                 if target_conn_args in kwargs.keys():
                     if kwargs[target_conn_args] is None:
@@ -152,7 +169,28 @@ class BaseSingleFactorBaseNode(object):
         return afunc
 
     def __call__(self, *sql, **kwargs):
-        return self.operator(*sql, **kwargs)
+        """
+
+        add delay function to delay execute sql
+
+        :param sql:
+        :param kwargs:
+        :return:
+        """
+
+        if 'delay' in kwargs.keys():
+            if kwargs['delay']:
+                if isinstance(sql, str):
+                    self.delay_tasks.append(sql)
+                else:
+                    for s in sql:
+                        self.delay_tasks.append(s)
+            else:
+                kwargs.pop('delay')
+                return self.operator(*sql, **kwargs)
+        else:
+
+            return self.operator(*sql, **kwargs)
 
     @property
     def __factor_id__(self):  # add iid function get factor table id
@@ -180,6 +218,11 @@ class BaseSingleFactorBaseNode(object):
         return res
 
     def _detect_complex_sql(self, warn=False):
+        """
+        detect complex sql
+        :param warn:
+        :return:
+        """
         sql = self.__sql__.lower()
         if 'select' in sql and len(sql.split('select ')) >= complex_sql_select_count:
             if warn:
@@ -346,80 +389,80 @@ class BaseSingleFactorBaseNode(object):
             return sql
 
     # update table
-    def __lshift__(self, src_db_table):
-        print('lshift')
-        fid_ck = self._fid_ck
-        dt_max_1st = self._dt_max_1st
-        execute = self._execute
-        no_self_update = self._no_self_update
-
-        if isinstance(src_db_table, str):
-            src_conn = copy.deepcopy(self.operator._src).replace(self.db_table, src_db_table)
-            src_db_table = BaseSingleFactorBaseNode(src_conn, cols=['*'])
-        elif isinstance(src_db_table, BaseSingleFactorBaseNode):
-            pass
-        else:
-            raise ValueError('src_db_table is not valid! please check!')
-
-        if src_db_table.empty:
-            raise ValueError(f'{src_db_table.db_table} is empty')
-        # check two table are same
-        if no_self_update and self.db_table == src_db_table.db_table and self.__factor_id__ == src_db_table.__factor_id__:
-            dst = src_db_table.db_table
-            src = self.db_table
-            raise ValueError(
-                f'Detect self-update process! these operator attempts to update data from {src} to {dst}')
-
-        update_status = 'full' if self.empty else 'incremental'
-
-        func = getattr(UpdateSQLUtils, f'{update_status}_update')
-        sql = func(src_db_table, self.db_table, fid_ck, dt_max_1st=dt_max_1st)
-        if execute:
-            self.operator(sql)
-        return sql, update_status
-
-    # update table
-    def __rshift__(self, dst_db_table: str):
-        """
-
-        UpdateSQLUtils
-
-        :param dst_db_table:
-        :return:
-        """
-        # print('rshift')
-        fid_ck = self._fid_ck
-        dt_max_1st = self._dt_max_1st
-        execute = self._execute
-        no_self_update = self._no_self_update
-        if self.empty:
-            raise ValueError(f'{self.db_table} is empty')
-
-        if isinstance(dst_db_table, str):
-            dst_conn = copy.deepcopy(self.operator._src)
-            dst_db_table = BaseSingleFactorBaseNode(
-                dst_conn.replace(self.db_table, dst_db_table),
-                cols=['*']
-            )
-        elif isinstance(dst_db_table, BaseSingleFactorBaseNode):
-            pass
-        else:
-            raise ValueError('dst_db_table is not valid! please check!')
-        # check two table are same
-        if no_self_update and self.db_table == dst_db_table.db_table:
-            if self.__factor_id__ == dst_db_table.__factor_id__:
-                dst = dst_db_table.db_table
-                src = self.db_table
-                raise ValueError(
-                    f'Detect self-update process! these operator attempts to update data from {src} to {dst}')
-
-        update_status = 'full' if dst_db_table.empty else 'incremental'
-
-        func = getattr(UpdateSQLUtils, f'{update_status}_update')
-        sql = func(self, dst_db_table, fid_ck, dt_max_1st=dt_max_1st)
-        if execute:
-            self.operator(sql)
-        return sql, update_status
+    # def __lshift__(self, src_db_table):
+    #     print('lshift')
+    #     fid_ck = self._fid_ck
+    #     dt_max_1st = self._dt_max_1st
+    #     execute = self._execute
+    #     no_self_update = self._no_self_update
+    #
+    #     if isinstance(src_db_table, str):
+    #         src_conn = copy.deepcopy(self.operator._src).replace(self.db_table, src_db_table)
+    #         src_db_table = BaseSingleQueryBaseNode(src_conn, cols=['*'])
+    #     elif isinstance(src_db_table, BaseSingleQueryBaseNode):
+    #         pass
+    #     else:
+    #         raise ValueError('src_db_table is not valid! please check!')
+    #
+    #     if src_db_table.empty:
+    #         raise ValueError(f'{src_db_table.db_table} is empty')
+    #     # check two table are same
+    #     if no_self_update and self.db_table == src_db_table.db_table and self.__factor_id__ == src_db_table.__factor_id__:
+    #         dst = src_db_table.db_table
+    #         src = self.db_table
+    #         raise ValueError(
+    #             f'Detect self-update process! these operator attempts to update data from {src} to {dst}')
+    #
+    #     update_status = 'full' if self.empty else 'incremental'
+    #
+    #     func = getattr(UpdateSQLUtils, f'{update_status}_update')
+    #     sql = func(src_db_table, self.db_table, fid_ck, dt_max_1st=dt_max_1st)
+    #     if execute:
+    #         self.operator(sql)
+    #     return sql, update_status
+    #
+    # # update table
+    # def __rshift__(self, dst_db_table: str):
+    #     """
+    #
+    #     UpdateSQLUtils
+    #
+    #     :param dst_db_table:
+    #     :return:
+    #     """
+    #     # print('rshift')
+    #     fid_ck = self._fid_ck
+    #     dt_max_1st = self._dt_max_1st
+    #     execute = self._execute
+    #     no_self_update = self._no_self_update
+    #     if self.empty:
+    #         raise ValueError(f'{self.db_table} is empty')
+    #
+    #     if isinstance(dst_db_table, str):
+    #         dst_conn = copy.deepcopy(self.operator._src)
+    #         dst_db_table = BaseSingleQueryBaseNode(
+    #             dst_conn.replace(self.db_table, dst_db_table),
+    #             cols=['*']
+    #         )
+    #     elif isinstance(dst_db_table, BaseSingleQueryBaseNode):
+    #         pass
+    #     else:
+    #         raise ValueError('dst_db_table is not valid! please check!')
+    #     # check two table are same
+    #     if no_self_update and self.db_table == dst_db_table.db_table:
+    #         if self.__factor_id__ == dst_db_table.__factor_id__:
+    #             dst = dst_db_table.db_table
+    #             src = self.db_table
+    #             raise ValueError(
+    #                 f'Detect self-update process! these operator attempts to update data from {src} to {dst}')
+    #
+    #     update_status = 'full' if dst_db_table.empty else 'incremental'
+    #
+    #     func = getattr(UpdateSQLUtils, f'{update_status}_update')
+    #     sql = func(self, dst_db_table, fid_ck, dt_max_1st=dt_max_1st)
+    #     if execute:
+    #         self.operator(sql)
+    #     return sql, update_status
 
 
 # class BaseSingleFactorNode(BaseSingleFactorBaseNode):
@@ -432,72 +475,75 @@ class BaseSingleFactorBaseNode(object):
 #         super(BaseSingleFactorNode, self).__init__(*args, **kwargs)
 
 
-class UpdateSQLUtils(object):
-
-    @staticmethod
-    def full_update(src_db_table: BaseSingleFactorBaseNode, dst_db_table: BaseSingleFactorBaseNode, **kwargs):
-        # dst_db_table = dst_db_table.db_table
-        # dst_db, dst_table = dst_db_table.db, dst_db_table.table
-        dst_table_type = dst_db_table.table_engine
-        dst = dst_db_table.db_table
-        if dst_table_type == 'View':
-            raise ValueError(f'{dst} is View ! cannot be updated!')
-        insert_sql = f"insert into {dst} {src_db_table}"
-        return insert_sql
-
-    @staticmethod
-    def incremental_update(src_db_table: BaseSingleFactorBaseNode, dst_db_table: BaseSingleFactorBaseNode,
-                           fid_ck: str, dt_max_1st=True, inplace=False, **kwargs):
-        # src_db_table = src_table.db_table
-        # src_table_type = src_db_table.table_engine
-        dst_table_type = dst_db_table.table_engine
-        dst = dst_db_table.db_table
-        if dst_table_type == 'View':
-            raise ValueError(f'{dst} is View ! cannot be updated!')
-        if dt_max_1st:
-            order_asc = ' desc'
-        else:
-            order_asc = ' asc'
-        sql = f" select distinct {fid_ck} from {dst} order by {fid_ck} {order_asc} limit 1 "
-        fid_ck_values = src_db_table.operator(sql).values.ravel().tolist()[0]
-        if inplace:
-            src_db_table._update(**{f'{fid_ck} as src_{fid_ck}': f' {fid_ck} > {fid_ck_values}'})
-            insert_sql = f"insert into {dst} {src_db_table}"
-        else:
-            src_db_table_copy = copy.deepcopy(src_db_table)
-            src_db_table_copy._update(**{f'{fid_ck} as src_{fid_ck}': f' {fid_ck} > {fid_ck_values}'})
-            insert_sql = f"insert into {dst} {src_db_table_copy}"
-
-        return insert_sql
+# class UpdateSQLUtils(object):
+#
+#     @staticmethod
+#     def full_update(src_db_table: BaseSingleQueryBaseNode, dst_db_table: BaseSingleQueryBaseNode, **kwargs):
+#         # dst_db_table = dst_db_table.db_table
+#         # dst_db, dst_table = dst_db_table.db, dst_db_table.table
+#         dst_table_type = dst_db_table.table_engine
+#         dst = dst_db_table.db_table
+#         if dst_table_type == 'View':
+#             raise ValueError(f'{dst} is View ! cannot be updated!')
+#         insert_sql = f"insert into {dst} {src_db_table}"
+#         return insert_sql
+#
+#     @staticmethod
+#     def incremental_update(src_db_table: BaseSingleQueryBaseNode, dst_db_table: BaseSingleQueryBaseNode,
+#                            fid_ck: str, dt_max_1st=True, inplace=False, **kwargs):
+#         # src_db_table = src_table.db_table
+#         # src_table_type = src_db_table.table_engine
+#         dst_table_type = dst_db_table.table_engine
+#         dst = dst_db_table.db_table
+#         if dst_table_type == 'View':
+#             raise ValueError(f'{dst} is View ! cannot be updated!')
+#         if dt_max_1st:
+#             order_asc = ' desc'
+#         else:
+#             order_asc = ' asc'
+#         sql = f" select distinct {fid_ck} from {dst} order by {fid_ck} {order_asc} limit 1 "
+#         fid_ck_values = src_db_table.operator(sql).values.ravel().tolist()[0]
+#         if inplace:
+#             src_db_table._update(**{f'{fid_ck} as src_{fid_ck}': f' {fid_ck} > {fid_ck_values}'})
+#             insert_sql = f"insert into {dst} {src_db_table}"
+#         else:
+#             src_db_table_copy = copy.deepcopy(src_db_table)
+#             src_db_table_copy._update(**{f'{fid_ck} as src_{fid_ck}': f' {fid_ck} > {fid_ck_values}'})
+#             insert_sql = f"insert into {dst} {src_db_table_copy}"
+#
+#         return insert_sql
 
 
-class BaseSingleFactorTableNode(BaseSingleFactorBaseNode):
+class BaseSingleFactorTableNode(BaseSingleQueryBaseNode):
     __slots__ = (
         'operator', 'db', 'table', 'db_table', '_kwargs', '_raw_kwargs', 'status', '_INFO', 'depend_tables',
-        '_fid_ck', '_dt_max_1st', '_execute', '_no_self_update', '_complex'
+        # '_fid_ck', '_dt_max_1st',
+        '_execute', '_no_self_update', '_complex'
     )
 
     def __init__(self, src: str, db_table: (None, str) = None, info=None,
-                 fid_ck: str = 'fid', dt_max_1st: bool = True, execute: bool = False,
-                 no_self_update: bool = True, **kwargs):
+                 # fid_ck: str = 'fid', dt_max_1st: bool = True,
+                 execute: bool = False,
+                 # no_self_update: bool = True,
+                 **kwargs):
         super(BaseSingleFactorTableNode, self).__init__(src, db_table=db_table, info=info, **kwargs)
         self._complex = False
-        self._fid_ck = fid_ck
-        self._dt_max_1st = dt_max_1st
+        # self._fid_ck = fid_ck
+        # self._dt_max_1st = dt_max_1st
         self._execute = execute
-        self._no_self_update = no_self_update
+        # self._no_self_update = no_self_update
 
-    def execute(self, sql):
-        ## add execute usage
-        return self.operator(sql)
-
-    def drop_table(self, target: str):
-        if '.' not in target:
-            raise ValueError('drop table must tell correspond database')
-        self.operator(f'drop table if exists {target}')
-
-    def drop_db(self, target: str):
-        self.operator(f'drop database if exists {target}')
+    # def execute(self, sql):
+    #     ## add execute usage
+    #     return self.operator(sql)
+    #
+    # def drop_table(self, target: str):
+    #     if '.' not in target:
+    #         raise ValueError('drop table must tell correspond database')
+    #     self.operator(f'drop table if exists {target}')
+    #
+    # def drop_db(self, target: str):
+    #     self.operator(f'drop database if exists {target}')
 
 
 ## https://zhuanlan.zhihu.com/p/297623539
